@@ -16,7 +16,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-from __future__ import print_function, division
 
 import time
 import os
@@ -118,7 +117,7 @@ def debug(level, msg, *args):
 
 
 def debug_lineno(*args):
-    """ Print an unformatted log message to stderr, contaning the line number
+    """ Print an unformatted log message to stderr, containing the line number
     """
     tb = traceback.extract_stack(limit=2)
     print((" %s:" "\033[01;33m"
@@ -437,7 +436,7 @@ class ReplayContext(object):
             for i in range(5):
                 p += ',DC'
                 if p != k and p in dn_map:
-                    print('dn_map collison %s %s' % (k, p),
+                    print('dn_map collision %s %s' % (k, p),
                           file=sys.stderr)
                     continue
                 dn_map[p] = dn_map[k]
@@ -998,7 +997,7 @@ class Conversation(object):
         if self.client_balance < 0:
             return (a, b)
 
-        # in the absense of a clue, we will fall through to assuming
+        # in the absence of a clue, we will fall through to assuming
         # the lowest number is the server (which is usually true).
 
         if self.client_balance == 0 and server_clue == b:
@@ -1007,7 +1006,7 @@ class Conversation(object):
         return (b, a)
 
     def forget_packets_outside_window(self, s, e):
-        """Prune any packets outside the timne window we're interested in
+        """Prune any packets outside the time window we're interested in
 
         :param s: start of the window
         :param e: end of the window
@@ -1189,7 +1188,9 @@ class TrafficModel(object):
         self.cumulative_duration = 0.0
         self.packet_rate = [0, 1]
 
-    def learn(self, conversations, dns_opcounts={}):
+    def learn(self, conversations, dns_opcounts=None):
+        if dns_opcounts is None:
+            dns_opcounts = {}
         prev = 0.0
         cum_duration = 0.0
         key = (NON_PACKET,) * (self.n - 1)
@@ -1589,7 +1590,6 @@ def replay_seq_in_fork(cs, start, context, account, client_id, server_id=1):
             os.close(1)
         except IOError as e:
             LOGGER.info("stdout closing failed with %s" % e)
-            pass
 
         sys.stdout = f
         now = time.time() - start
@@ -1631,7 +1631,6 @@ def dnshammer_in_fork(dns_rate, duration, context, query_file=None):
         os.close(1)
     except IOError as e:
         LOGGER.warn("stdout closing failed with %s" % e)
-        pass
     filename = os.path.join(context.statsdir, 'stats-dns')
     sys.stdout = open(filename, 'w')
 
@@ -1925,7 +1924,7 @@ def user_name(instance_id, i):
 
 
 def search_objectclass(ldb, objectclass='user', attr='sAMAccountName'):
-    """Seach objectclass, return attr in a set"""
+    """Search objectclass, return attr in a set"""
     objs = ldb.search(
         expression="(objectClass={})".format(objectclass),
         attrs=[attr]
