@@ -206,7 +206,7 @@ def set_admin_password(logger, samdb):
 
 
 class cmd_domain_backup_online(samba.netcmd.Command):
-    '''Copy a running DC's current DB into a backup tar file.
+    """Copy a running DC's current DB into a backup tar file.
 
     Takes a backup copy of the current domain from a running DC. If the domain
     were to undergo a catastrophic failure, then the backup file can be used to
@@ -219,7 +219,7 @@ class cmd_domain_backup_online(samba.netcmd.Command):
     - all the domain's secrets are included in the backup file.
     - although the DB contents can be untarred and examined manually, you need
       to run 'samba-tool domain backup restore' before you can start a Samba DC
-      from the backup file.'''
+      from the backup file."""
 
     synopsis = "%prog --server=<DC-to-backup> --targetdir=<output-dir>"
     takes_optiongroups = {
@@ -304,7 +304,7 @@ class cmd_domain_backup_online(samba.netcmd.Command):
 
 
 class cmd_domain_backup_restore(cmd_fsmo_seize):
-    '''Restore the domain's DB from a backup-file.
+    """Restore the domain's DB from a backup-file.
 
     This restores a previously backed up copy of the domain's DB on a new DC.
 
@@ -317,7 +317,7 @@ class cmd_domain_backup_restore(cmd_fsmo_seize):
     be joined to the new DC to recover the network.
 
     Note that this command should be run as the root user - it will fail
-    otherwise.'''
+    otherwise."""
 
     synopsis = ("%prog --backup-file=<tar-file> --targetdir=<output-dir> "
                 "--newservername=<DC-name>")
@@ -339,10 +339,10 @@ class cmd_domain_backup_restore(cmd_fsmo_seize):
 
     def register_dns_zone(self, logger, samdb, lp, ntdsguid, host_ip,
                           host_ip6, site):
-        '''
+        """
         Registers the new realm's DNS objects when a renamed domain backup
         is restored.
-        '''
+        """
         names = guess_names(lp)
         domaindn = names.domaindn
         forestdn = samdb.get_root_basedn().get_linearized()
@@ -371,7 +371,7 @@ class cmd_domain_backup_restore(cmd_fsmo_seize):
                                  dnsadmins_sid, add_root=False)
 
     def fix_old_dc_references(self, samdb):
-        '''Fixes attributes that reference the old/removed DCs'''
+        """Fixes attributes that reference the old/removed DCs"""
 
         # we just want to fix up DB problems here that were introduced by us
         # removing the old DCs. We restrict what we fix up so that the restored
@@ -396,7 +396,7 @@ class cmd_domain_backup_restore(cmd_fsmo_seize):
         samdb.transaction_commit()
 
     def create_default_site(self, samdb, logger):
-        '''Creates the default site, if it doesn't already exist'''
+        """Creates the default site, if it doesn't already exist"""
 
         sitename = DEFAULTSITE
         search_expr = "(&(cn={0})(objectclass=site))".format(sitename)
@@ -692,7 +692,7 @@ class cmd_domain_backup_restore(cmd_fsmo_seize):
 
 
 class cmd_domain_backup_rename(samba.netcmd.Command):
-    '''Copy a running DC's DB to backup file, renaming the domain in the process.
+    """Copy a running DC's DB to backup file, renaming the domain in the process.
 
     Where <new-domain> is the new domain's NetBIOS name, and <new-dnsrealm> is
     the new domain's realm in DNS form.
@@ -718,7 +718,7 @@ class cmd_domain_backup_rename(samba.netcmd.Command):
       in order to work (they will still refer to the old DC's IP instead of the
       new DC's address).
     - we recommend that you only use this option if you know what you're doing.
-    '''
+    """
 
     synopsis = ("%prog <new-domain> <new-dnsrealm> --server=<DC-to-backup> "
                 "--targetdir=<output-dir>")
@@ -744,7 +744,7 @@ class cmd_domain_backup_rename(samba.netcmd.Command):
     takes_args = ["new_domain_name", "new_dns_realm"]
 
     def update_dns_root(self, logger, samdb, old_realm, delete_old_dns):
-        '''Updates dnsRoot for the partition objects to reflect the rename'''
+        """Updates dnsRoot for the partition objects to reflect the rename"""
 
         # lookup the crossRef objects that hold the old realm's dnsRoot
         partitions_dn = samdb.get_partitions_dn()
@@ -783,7 +783,7 @@ class cmd_domain_backup_rename(samba.netcmd.Command):
     # Updates the CN=<domain>,CN=Partitions,CN=Configuration,... object to
     # reflect the domain rename
     def rename_domain_partition(self, logger, samdb, new_netbios_name):
-        '''Renames the domain partition object and updates its nETBIOSName'''
+        """Renames the domain partition object and updates its nETBIOSName"""
 
         # lookup the crossRef object that holds the nETBIOSName (nCName has
         # already been updated by this point, but the netBIOS hasn't)
@@ -822,7 +822,7 @@ class cmd_domain_backup_rename(samba.netcmd.Command):
         samdb.delete(dn, ["tree_delete:1"])
 
     def fix_old_dn_attributes(self, samdb):
-        '''Fixes attributes (i.e. objectCategory) that still use the old DN'''
+        """Fixes attributes (i.e. objectCategory) that still use the old DN"""
 
         samdb.transaction_start()
         # Just fix any mismatches in DN detected (leave any other errors)
@@ -942,7 +942,7 @@ class cmd_domain_backup_rename(samba.netcmd.Command):
 
 
 class cmd_domain_backup_offline(samba.netcmd.Command):
-    '''Backup the local domain directories safely into a tar file.
+    """Backup the local domain directories safely into a tar file.
 
     Takes a backup copy of the current domain from the local files on disk,
     with proper locking of the DB to ensure consistency. If the domain were to
@@ -953,7 +953,7 @@ class cmd_domain_backup_offline(samba.netcmd.Command):
     - a backup can be created even if the DC isn't currently running.
     - includes non-replicated attributes that an online backup wouldn't store.
     - takes a copy of the raw database files, which has the risk that any
-      hidden problems in the DB are preserved in the backup.'''
+      hidden problems in the DB are preserved in the backup."""
 
     synopsis = "%prog [options]"
     takes_optiongroups = {
@@ -1249,7 +1249,7 @@ class cmd_domain_backup_offline(samba.netcmd.Command):
 
 
 class cmd_domain_backup(samba.netcmd.SuperCommand):
-    '''Create or restore a backup of the domain.'''
+    """Create or restore a backup of the domain."""
     subcommands = {'offline': cmd_domain_backup_offline(),
                    'online': cmd_domain_backup_online(),
                    'rename': cmd_domain_backup_rename(),
