@@ -34,6 +34,7 @@ from samba.dcerpc import drsblobs, misc
 from samba.common import normalise_int32
 from samba.common import get_bytes, cmp
 from samba.dcerpc import security
+from samba import is_ad_dc_built
 import binascii
 
 __docformat__ = "restructuredText"
@@ -54,6 +55,26 @@ class SamDB(samba.Ldb):
     hash_oid_name = {}
     hash_well_known = {}
 
+    class _CleanUpOnError:
+        def __init__(self, samdb, dn):
+            self.samdb = samdb
+            self.dn = dn
+
+        def __enter__(self):
+            pass
+
+        def __exit__(self, exc_type, exc_val, exc_tb):
+            if exc_type is not None:
+                # We failed to modify the account. If we connected to the
+                # database over LDAP, we don't have transactions, and so when
+                # we call transaction_cancel(), the account will still exist in
+                # a half-created state. We'll delete the account to ensure that
+                # doesn't happen.
+                self.samdb.delete(self.dn)
+
+            # Don't suppress any exceptions
+            return False
+
     def __init__(self, url=None, lp=None, modules_dir=None, session_info=None,
                  credentials=None, flags=ldb.FLG_DONT_CREATE_DB,
                  options=None, global_schema=True,
@@ -66,9 +87,9 @@ class SamDB(samba.Ldb):
 
         self.url = url
 
-        super(SamDB, self).__init__(url=url, lp=lp, modules_dir=modules_dir,
-                                    session_info=session_info, credentials=credentials, flags=flags,
-                                    options=options)
+        super().__init__(url=url, lp=lp, modules_dir=modules_dir,
+                         session_info=session_info, credentials=credentials, flags=flags,
+                         options=options)
 
         if global_schema:
             dsdb._dsdb_set_global_schema(self)
@@ -77,28 +98,27 @@ class SamDB(samba.Ldb):
             dsdb._dsdb_set_am_rodc(self, am_rodc)
 
     def connect(self, url=None, flags=0, options=None):
-        '''connect to the database'''
+        """connect to the database"""
         if self.lp is not None and not os.path.exists(url):
             url = self.lp.private_path(url)
         self.url = url
 
-        super(SamDB, self).connect(url=url, flags=flags,
-                                   options=options)
+        super().connect(url=url, flags=flags, options=options)
 
     def am_rodc(self):
-        '''return True if we are an RODC'''
+        """return True if we are an RODC"""
         return dsdb._am_rodc(self)
 
     def am_pdc(self):
-        '''return True if we are an PDC emulator'''
+        """return True if we are an PDC emulator"""
         return dsdb._am_pdc(self)
 
     def domain_dn(self):
-        '''return the domain DN'''
+        """return the domain DN"""
         return str(self.get_default_basedn())
 
     def schema_dn(self):
-        '''return the schema partition dn'''
+        """return the schema partition dn"""
         return str(self.get_schema_basedn())
 
     def disable_account(self, search_filter):
@@ -227,7 +247,12 @@ lockoutTime: 0
         :param sd: security descriptor of the object
         """
 
-        group_dn = "CN=%s,%s,%s" % (groupname, (groupou or "CN=Users"), self.domain_dn())
+        if groupou:
+            group_dn = "CN=%s,%s,%s" % (groupname, groupou, self.domain_dn())
+        else:
+            group_dn = "CN=%s,%s" % (groupname, self.get_wellknown_dn(
+                                        self.get_default_basedn(),
+                                        dsdb.DS_GUID_USERS_CONTAINER))
 
         # The new user record. Note the reliance on the SAMLDB module which
         # fills in the default information
@@ -335,7 +360,7 @@ lockoutTime: 0
 
     def add_remove_group_members(self, groupname, members,
                                  add_members_operation=True,
-                                 member_types=[ 'user', 'group', 'computer' ],
+                                 member_types=None,
                                  member_base_dn=None):
         """Adds or removes group members
 
@@ -344,6 +369,8 @@ lockoutTime: 0
         :param add_members_operation: Defines if its an add or remove
             operation
         """
+        if member_types is None:
+            member_types = ['user', 'group', 'computer']
 
         groupfilter = "(&(sAMAccountName=%s)(objectCategory=%s,%s))" % (
             ldb.binary_encode(groupname), "CN=Group,CN=Schema,CN=Configuration", self.domain_dn())
@@ -371,7 +398,7 @@ changetype: modify
                 try:
                     membersid = security.dom_sid(member)
                     targetmember_dn = "<SID=%s>" % str(membersid)
-                except TypeError as e:
+                except ValueError:
                     pass
 
                 if targetmember_dn is None:
@@ -447,10 +474,12 @@ member: %s
         msg.add(el)
 
     def fullname_from_names(self, given_name=None, initials=None, surname=None,
-                            old_attrs={}, fallback_default=""):
+                            old_attrs=None, fallback_default=""):
         """Prepares new combined fullname, using the name parts.
         Used for things like displayName or cn.
         Use the original name values, if no new one is specified."""
+        if old_attrs is None:
+            old_attrs = {}
 
         attrs = {"givenName": given_name,
                  "initials": initials,
@@ -458,10 +487,10 @@ member: %s
 
         # if the attribute is not specified, try to use the old one
         for attr_name, attr_value in attrs.items():
-            if attr_value == None and attr_name in old_attrs:
+            if attr_value is None and attr_name in old_attrs:
                 attrs[attr_name] = str(old_attrs[attr_name])
 
-        # add '.' to initials if initals are not None and not "" and if the initials
+        # add '.' to initials if initials are not None and not "" and if the initials
         # don't have already a '.' at the end
         if attrs["initials"] and not attrs["initials"].endswith('.'):
             attrs["initials"] += '.'
@@ -529,7 +558,12 @@ member: %s
         if useusernameascn is None and displayname != "":
             cn = displayname
 
-        user_dn = "CN=%s,%s,%s" % (cn, (userou or "CN=Users"), self.domain_dn())
+        if userou:
+            user_dn = "CN=%s,%s,%s" % (cn, userou, self.domain_dn())
+        else:
+            user_dn = "CN=%s,%s" % (cn, self.get_wellknown_dn(
+                                        self.get_default_basedn(),
+                                        dsdb.DS_GUID_USERS_CONTAINER))
 
         dnsdomain = ldb.Dn(self, self.domain_dn()).canonical_str().replace("/", "")
         user_principal_name = "%s@%s" % (username, dnsdomain)
@@ -627,15 +661,17 @@ member: %s
         self.transaction_start()
         try:
             self.add(ldbmessage)
-            if ldbmessage2:
-                self.modify(ldbmessage2)
 
-            # Sets the password for it
-            if setpassword:
-                self.setpassword(("(distinguishedName=%s)" %
-                                  ldb.binary_encode(user_dn)),
-                                 password,
-                                 force_password_change_at_next_login_req)
+            with self._CleanUpOnError(self, user_dn):
+                if ldbmessage2:
+                    self.modify(ldbmessage2)
+
+                # Sets the password for it
+                if setpassword:
+                    self.setpassword(("(distinguishedName=%s)" %
+                                      ldb.binary_encode(user_dn)),
+                                     password,
+                                     force_password_change_at_next_login_req)
         except:
             self.transaction_cancel()
             raise
@@ -762,7 +798,8 @@ member: %s
             raise Exception('Illegal computername "%s"' % computername)
         samaccountname = "%s$" % cn
 
-        computercontainer_dn = "CN=Computers,%s" % self.domain_dn()
+        computercontainer_dn = self.get_wellknown_dn(self.get_default_basedn(),
+                                              dsdb.DS_GUID_COMPUTERS_CONTAINER)
         if computerou:
             computercontainer_dn = self.normalize_dn_in_domain(computerou)
 
@@ -795,9 +832,10 @@ member: %s
 
             if prepare_oldjoin:
                 password = cn.lower()
-                self.setpassword(("(distinguishedName=%s)" %
-                                  ldb.binary_encode(computer_dn)),
-                                 password, False)
+                with self._CleanUpOnError(self, computer_dn):
+                    self.setpassword(("(distinguishedName=%s)" %
+                                      ldb.binary_encode(computer_dn)),
+                                     password, False)
         except:
             self.transaction_cancel()
             raise
@@ -888,7 +926,6 @@ unicodePwd:: %s
             user_dn = res[0].dn
 
             userAccountControl = int(res[0]["userAccountControl"][0])
-            accountExpires     = int(res[0]["accountExpires"][0])
             if no_expiry_req:
                 userAccountControl = userAccountControl | 0x10000
                 accountExpires = 0
@@ -945,29 +982,29 @@ accountExpires: %u
 
     def get_attid_from_lDAPDisplayName(self, ldap_display_name,
                                        is_schema_nc=False):
-        '''return the attribute ID for a LDAP attribute as an integer as found in DRSUAPI'''
+        """return the attribute ID for a LDAP attribute as an integer as found in DRSUAPI"""
         return dsdb._dsdb_get_attid_from_lDAPDisplayName(self,
                                                          ldap_display_name, is_schema_nc)
 
     def get_syntax_oid_from_lDAPDisplayName(self, ldap_display_name):
-        '''return the syntax OID for a LDAP attribute as a string'''
+        """return the syntax OID for a LDAP attribute as a string"""
         return dsdb._dsdb_get_syntax_oid_from_lDAPDisplayName(self, ldap_display_name)
 
     def get_systemFlags_from_lDAPDisplayName(self, ldap_display_name):
-        '''return the systemFlags for a LDAP attribute as a integer'''
+        """return the systemFlags for a LDAP attribute as a integer"""
         return dsdb._dsdb_get_systemFlags_from_lDAPDisplayName(self, ldap_display_name)
 
     def get_linkId_from_lDAPDisplayName(self, ldap_display_name):
-        '''return the linkID for a LDAP attribute as a integer'''
+        """return the linkID for a LDAP attribute as a integer"""
         return dsdb._dsdb_get_linkId_from_lDAPDisplayName(self, ldap_display_name)
 
     def get_lDAPDisplayName_by_attid(self, attid):
-        '''return the lDAPDisplayName from an integer DRS attribute ID'''
+        """return the lDAPDisplayName from an integer DRS attribute ID"""
         return dsdb._dsdb_get_lDAPDisplayName_by_attid(self, attid)
 
     def get_backlink_from_lDAPDisplayName(self, ldap_display_name):
-        '''return the attribute name of the corresponding backlink from the name
-        of a forward link attribute. If there is no backlink return None'''
+        """return the attribute name of the corresponding backlink from the name
+        of a forward link attribute. If there is no backlink return None"""
         return dsdb._dsdb_get_backlink_from_lDAPDisplayName(self, ldap_display_name)
 
     def set_ntds_settings_dn(self, ntds_settings_dn):
@@ -1052,11 +1089,11 @@ schemaUpdateNow: 1
         self.modify_ldif(ldif)
 
     def dsdb_DsReplicaAttribute(self, ldb, ldap_display_name, ldif_elements):
-        '''convert a list of attribute values to a DRSUAPI DsReplicaAttribute'''
+        """convert a list of attribute values to a DRSUAPI DsReplicaAttribute"""
         return dsdb._dsdb_DsReplicaAttribute(ldb, ldap_display_name, ldif_elements)
 
     def dsdb_normalise_attributes(self, ldb, ldap_display_name, ldif_elements):
-        '''normalise a list of attribute values'''
+        """normalise a list of attribute values"""
         return dsdb._dsdb_normalise_attributes(ldb, ldap_display_name, ldif_elements)
 
     def get_attribute_from_attid(self, attid):
@@ -1297,7 +1334,7 @@ schemaUpdateNow: 1
         """Creates an organizationalUnit object
         :param ou_dn: dn of the new object
         :param description: description attribute
-        :param name: name atttribute
+        :param name: name attribute
         :param sd: security descriptor of the object, can be
         an SDDL string or security.descriptor type
         """
@@ -1319,7 +1356,7 @@ schemaUpdateNow: 1
          """
         self.transaction_start()
         try:
-            seq = super(SamDB, self).sequence_number(seq_type)
+            seq = super().sequence_number(seq_type)
         except:
             self.transaction_cancel()
             raise
@@ -1328,17 +1365,17 @@ schemaUpdateNow: 1
         return seq
 
     def get_dsServiceName(self):
-        '''get the NTDS DN from the rootDSE'''
+        """get the NTDS DN from the rootDSE"""
         res = self.search(base="", scope=ldb.SCOPE_BASE, attrs=["dsServiceName"])
         return str(res[0]["dsServiceName"][0])
 
     def get_serverName(self):
-        '''get the server DN from the rootDSE'''
+        """get the server DN from the rootDSE"""
         res = self.search(base="", scope=ldb.SCOPE_BASE, attrs=["serverName"])
         return str(res[0]["serverName"][0])
 
     def dns_lookup(self, dns_name, dns_partition=None):
-        '''Do a DNS lookup in the database, returns the NDR database structures'''
+        """Do a DNS lookup in the database, returns the NDR database structures"""
         if dns_partition is None:
             return dsdb_dns.lookup(self, dns_name)
         else:
@@ -1346,28 +1383,32 @@ schemaUpdateNow: 1
                                    dns_partition=dns_partition)
 
     def dns_extract(self, el):
-        '''Return the NDR database structures from a dnsRecord element'''
+        """Return the NDR database structures from a dnsRecord element"""
         return dsdb_dns.extract(self, el)
 
     def dns_replace(self, dns_name, new_records):
-        '''Do a DNS modification on the database, sets the NDR database
+        """Do a DNS modification on the database, sets the NDR database
         structures on a DNS name
-        '''
+        """
         return dsdb_dns.replace(self, dns_name, new_records)
 
     def dns_replace_by_dn(self, dn, new_records):
-        '''Do a DNS modification on the database, sets the NDR database
+        """Do a DNS modification on the database, sets the NDR database
         structures on a LDB DN
 
         This routine is important because if the last record on the DN
         is removed, this routine will put a tombstone in the record.
-        '''
+        """
         return dsdb_dns.replace_by_dn(self, dn, new_records)
 
     def garbage_collect_tombstones(self, dn, current_time,
                                    tombstone_lifetime=None):
-        '''garbage_collect_tombstones(lp, samdb, [dn], current_time, tombstone_lifetime)
-        -> (num_objects_expunged, num_links_expunged)'''
+        """garbage_collect_tombstones(lp, samdb, [dn], current_time, tombstone_lifetime)
+        -> (num_objects_expunged, num_links_expunged)"""
+
+        if not is_ad_dc_built():
+            raise SamDBError('Cannot garbage collect tombstones: '
+                'AD DC was not built')
 
         if tombstone_lifetime is None:
             return dsdb._dsdb_garbage_collect_tombstones(self, dn,
@@ -1378,33 +1419,33 @@ schemaUpdateNow: 1
                                                          tombstone_lifetime)
 
     def create_own_rid_set(self):
-        '''create a RID set for this DSA'''
+        """create a RID set for this DSA"""
         return dsdb._dsdb_create_own_rid_set(self)
 
     def allocate_rid(self):
-        '''return a new RID from the RID Pool on this DSA'''
+        """return a new RID from the RID Pool on this DSA"""
         return dsdb._dsdb_allocate_rid(self)
 
     def next_free_rid(self):
-        '''return the next free RID from the RID Pool on this DSA.
+        """return the next free RID from the RID Pool on this DSA.
 
         :note: This function is not intended for general use, and care must be
             taken if it is used to generate objectSIDs. The returned RID is not
             formally reserved for use, creating the possibility of duplicate
             objectSIDs.
-        '''
+        """
         rid, _ = self.free_rid_bounds()
         return rid
 
     def free_rid_bounds(self):
-        '''return the low and high bounds (inclusive) of RIDs that are
+        """return the low and high bounds (inclusive) of RIDs that are
             available for use in this DSA's current RID pool.
 
         :note: This function is not intended for general use, and care must be
             taken if it is used to generate objectSIDs. The returned range of
             RIDs is not formally reserved for use, creating the possibility of
             duplicate objectSIDs.
-        '''
+        """
         # Get DN of this server's RID Set
         server_name_dn = ldb.Dn(self, self.get_serverName())
         res = self.search(base=server_name_dn,
@@ -1464,8 +1505,8 @@ schemaUpdateNow: 1
         if prev_pool == uint64_max or next_rid == uint32_max:
             prev_pool = alloc_pool
             next_rid = prev_pool & uint32_max
-
-        next_rid += 1
+        else:
+            next_rid += 1
 
         # Now check if our current pool is still usable
         prev_pool_lo = prev_pool & uint32_max
@@ -1491,13 +1532,13 @@ schemaUpdateNow: 1
         return next_rid, prev_pool_hi
 
     def normalize_dn_in_domain(self, dn):
-        '''return a new DN expanded by adding the domain DN
+        """return a new DN expanded by adding the domain DN
 
         If the dn is already a child of the domain DN, just
         return it as-is.
 
         :param dn: relative dn
-        '''
+        """
         domain_dn = ldb.Dn(self, self.domain_dn())
 
         if isinstance(dn, ldb.Dn):
@@ -1509,10 +1550,10 @@ schemaUpdateNow: 1
         return full_dn
 
 class dsdb_Dn(object):
-    '''a class for binary DN'''
+    """a class for binary DN"""
 
     def __init__(self, samdb, dnstring, syntax_oid=None):
-        '''create a dsdb_Dn'''
+        """create a dsdb_Dn"""
         if syntax_oid is None:
             # auto-detect based on string
             if dnstring.startswith("B:"):
@@ -1540,7 +1581,7 @@ class dsdb_Dn(object):
         return self.prefix + str(self.dn.extended_str(mode=1))
 
     def __cmp__(self, other):
-        ''' compare dsdb_Dn values similar to parsed_dn_compare()'''
+        """ compare dsdb_Dn values similar to parsed_dn_compare()"""
         dn1 = self
         dn2 = other
         guid1 = dn1.dn.get_extended_component("GUID")
@@ -1572,11 +1613,11 @@ class dsdb_Dn(object):
         return self.__cmp__(other) >= 0
 
     def get_binary_integer(self):
-        '''return binary part of a dsdb_Dn as an integer, or None'''
+        """return binary part of a dsdb_Dn as an integer, or None"""
         if self.prefix == '':
             return None
         return int(self.binary, 16)
 
     def get_bytes(self):
-        '''return binary as a byte string'''
+        """return binary as a byte string"""
         return binascii.unhexlify(self.binary)
