@@ -19,20 +19,20 @@
 """Samba Python tests."""
 import os
 import tempfile
-import warnings
+import traceback
 import collections
 import ldb
 import samba
 from samba import param
 from samba import credentials
 from samba.credentials import Credentials
-from samba import gensec
 import subprocess
 import sys
 import unittest
 import re
 from enum import IntEnum, unique
 import samba.auth
+import samba.gensec
 import samba.dcerpc.base
 from random import randint
 from random import SystemRandom
@@ -59,7 +59,10 @@ BINDIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
 
 HEXDUMP_FILTER = bytearray([x if ((len(repr(chr(x))) == 3) and (x < 127)) else ord('.') for x in range(256)])
 
-LDB_ERR_LUT = {v: k for k,v in vars(ldb).items() if k.startswith('ERR_')}
+LDB_ERR_LUT = {v: k for k, v in vars(ldb).items() if k.startswith('ERR_')}
+
+RE_CAMELCASE = re.compile(r"([_\-])+")
+
 
 def ldb_err(v):
     if isinstance(v, ldb.LdbError):
@@ -79,9 +82,63 @@ def DynamicTestCase(cls):
     cls.setUpDynamicTestCases()
     return cls
 
+
 class TestCase(unittest.TestCase):
     """A Samba test case."""
 
+    # Re-implement addClassCleanup to support Python versions older than 3.8.
+    # Can be removed once these older Python versions are no longer needed.
+    if sys.version_info.major == 3 and sys.version_info.minor < 8:
+        _class_cleanups = []
+
+        @classmethod
+        def addClassCleanup(cls, function, *args, **kwargs):
+            cls._class_cleanups.append((function, args, kwargs))
+
+        @classmethod
+        def tearDownClass(cls):
+            teardown_exceptions = []
+
+            while cls._class_cleanups:
+                function, args, kwargs = cls._class_cleanups.pop()
+                try:
+                    function(*args, **kwargs)
+                except Exception:
+                    teardown_exceptions.append(traceback.format_exc())
+
+            # ExceptionGroup would be better but requires Python 3.11
+            if teardown_exceptions:
+                raise ValueError("tearDownClass failed:\n\n" +
+                                 "\n".join(teardown_exceptions))
+
+        @classmethod
+        def setUpClass(cls):
+            """
+            Call setUpTestData, ensure tearDownClass is called on exceptions.
+
+            This is only required on Python versions older than 3.8.
+            """
+            try:
+                cls.setUpTestData()
+            except Exception:
+                cls.tearDownClass()
+                raise
+    else:
+        @classmethod
+        def setUpClass(cls):
+            """
+            setUpClass only needs to call setUpTestData.
+
+            On Python 3.8 and above unittest will always call tearDownClass,
+            even if an exception was raised in setUpClass.
+            """
+            cls.setUpTestData()
+
+    @classmethod
+    def setUpTestData(cls):
+        """Create class level test fixtures here."""
+        pass
+
     @classmethod
     def generate_dynamic_test(cls, fnname, suffix, *args, doc=None):
         """
@@ -108,10 +165,27 @@ class TestCase(unittest.TestCase):
         In order to implement autogenerated testcase permutations.
         """
         msg = "%s needs setUpDynamicTestCases() if @DynamicTestCase is used!" % (cls)
-        raise Exception(msg)
+        raise NotImplementedError(msg)
+
+    def unique_name(self):
+        """Generate a unique name from within a test for creating objects.
+
+        Used to ensure each test generates uniquely named objects that don't
+        interfere with other tests.
+        """
+        # name of calling function
+        name = self.id().rsplit(".", 1)[1]
+
+        # remove test_ prefix
+        if name.startswith("test_"):
+            name = name[5:]
+
+        # finally, convert to camelcase
+        name = RE_CAMELCASE.sub(" ", name).title().replace(" ", "")
+        return "".join([name[0].lower(), name[1:]])
 
     def setUp(self):
-        super(TestCase, self).setUp()
+        super().setUp()
         test_debug_level = os.getenv("TEST_DEBUG_LEVEL")
         if test_debug_level is not None:
             test_debug_level = int(test_debug_level)
@@ -119,12 +193,32 @@ class TestCase(unittest.TestCase):
             samba.set_debug_level(test_debug_level)
             self.addCleanup(samba.set_debug_level, test_debug_level)
 
-    def get_loadparm(self):
-        return env_loadparm()
+    @classmethod
+    def get_loadparm(cls, s3=False):
+        return env_loadparm(s3=s3)
 
     def get_credentials(self):
         return cmdline_credentials
 
+    @classmethod
+    def get_env_credentials(cls, *, lp, env_username, env_password,
+                            env_realm=None, env_domain=None):
+        creds = credentials.Credentials()
+
+        # guess Credentials parameters here. Otherwise, workstation
+        # and domain fields are NULL and gencache code segfaults
+        creds.guess(lp)
+        creds.set_username(env_get_var_value(env_username))
+        creds.set_password(env_get_var_value(env_password))
+
+        if env_realm is not None:
+            creds.set_realm(env_get_var_value(env_realm))
+
+        if env_domain is not None:
+            creds.set_domain(env_get_var_value(env_domain))
+
+        return creds
+
     def get_creds_ccache_name(self):
         creds = self.get_credentials()
         ccache = creds.get_named_ccache(self.get_loadparm())
@@ -174,7 +268,7 @@ class TestCase(unittest.TestCase):
         if kerberos_state is None:
             kerberos_state = template.get_kerberos_state()
 
-        # get a copy of the global creds or a the passed in creds
+        # get a copy of the global creds or the passed in creds
         c = Credentials()
         c.set_username(username)
         c.set_password(userpass)
@@ -182,7 +276,7 @@ class TestCase(unittest.TestCase):
         c.set_realm(template.get_realm())
         c.set_workstation(template.get_workstation())
         c.set_gensec_features(c.get_gensec_features()
-                              | gensec.FEATURE_SEAL)
+                              | samba.gensec.FEATURE_SEAL)
         c.set_kerberos_state(kerberos_state)
         if simple_bind_dn:
             c.set_bind_dn(simple_bind_dn)
@@ -252,7 +346,7 @@ class LdbTestCase(TestCase):
     """Trivial test case for running tests against a LDB."""
 
     def setUp(self):
-        super(LdbTestCase, self).setUp()
+        super().setUp()
         self.tempfile = tempfile.NamedTemporaryFile(delete=False)
         self.filename = self.tempfile.name
         self.ldb = samba.Ldb(self.filename)
@@ -271,7 +365,7 @@ class LdbTestCase(TestCase):
 class TestCaseInTempDir(TestCase):
 
     def setUp(self):
-        super(TestCaseInTempDir, self).setUp()
+        super().setUp()
         self.tempdir = tempfile.mkdtemp()
         self.addCleanup(self._remove_tempdir)
 
@@ -311,7 +405,7 @@ class TestCaseInTempDir(TestCase):
             # os.path.join will happily step out of the tempdir,
             # so let's just check.
             if os.path.dirname(path) != self.tempdir:
-                raise ValueError("{path} might be outside {self.tempdir}")
+                raise ValueError(f"{path} might be outside {self.tempdir}")
 
             try:
                 _rm(path)
@@ -330,15 +424,19 @@ class TestCaseInTempDir(TestCase):
         self.rm_files(*dirs, allow_missing=allow_missing, _rm=shutil.rmtree)
 
 
-def env_loadparm():
-    lp = param.LoadParm()
+def env_loadparm(s3=False):
+    if s3:
+        from samba.samba3 import param as s3param
+        lp = s3param.get_context()
+    else:
+        lp = param.LoadParm()
+
     try:
         lp.load(os.environ["SMB_CONF_PATH"])
     except KeyError:
         raise KeyError("SMB_CONF_PATH not set")
     return lp
 
-
 def env_get_var_value(var_name, allow_missing=False):
     """Returns value for variable in os.environ
 
@@ -405,14 +503,14 @@ class BlackboxTestCase(TestCaseInTempDir):
         exe = os.path.join(BINDIR, cmd)
 
         python_cmds = ["samba-tool",
-            "samba_dnsupdate",
-            "samba_upgradedns",
-            "script/traffic_replay",
-            "script/traffic_learner"]
+                       "samba_dnsupdate",
+                       "samba_upgradedns",
+                       "script/traffic_replay",
+                       "script/traffic_learner"]
 
         if os.path.exists(exe):
             parts[0] = exe
-        if cmd in python_cmds and os.getenv("PYTHON", None):
+        if cmd in python_cmds and os.getenv("PYTHON", False):
             parts.insert(0, os.environ["PYTHON"])
 
         if not isinstance(line, list):
@@ -420,11 +518,13 @@ class BlackboxTestCase(TestCaseInTempDir):
 
         return line
 
-    def check_run(self, line, msg=None):
-        self.check_exit_code(line, 0, msg=msg)
+    @classmethod
+    def check_run(cls, line, msg=None):
+        cls.check_exit_code(line, 0, msg=msg)
 
-    def check_exit_code(self, line, expected, msg=None):
-        line = self._make_cmdline(line)
+    @classmethod
+    def check_exit_code(cls, line, expected, msg=None):
+        line = cls._make_cmdline(line)
         use_shell = not isinstance(line, list)
         p = subprocess.Popen(line,
                              stdout=subprocess.PIPE,
@@ -442,9 +542,10 @@ class BlackboxTestCase(TestCaseInTempDir):
                                        msg)
         return stdoutdata
 
-    def check_output(self, line):
+    @classmethod
+    def check_output(cls, line):
         use_shell = not isinstance(line, list)
-        line = self._make_cmdline(line)
+        line = cls._make_cmdline(line)
         p = subprocess.Popen(line, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                              shell=use_shell, close_fds=True)
         stdoutdata, stderrdata = p.communicate()
@@ -478,12 +579,12 @@ class BlackboxTestCase(TestCaseInTempDir):
         password += SystemRandom().choice(string.digits)
         password += SystemRandom().choice(string.ascii_lowercase)
         password += ''.join(SystemRandom().choice(string.ascii_uppercase +
-                    string.ascii_lowercase +
-                    string.digits) for x in range(count - 3))
+                            string.ascii_lowercase +
+                            string.digits) for x in range(count - 3))
         return password
 
 
-def connect_samdb(samdb_url, lp=None, session_info=None, credentials=None,
+def connect_samdb(samdb_url, *, lp=None, session_info=None, credentials=None,
                   flags=0, ldb_options=None, ldap_only=False, global_schema=True):
     """Create SamDB instance and connects to samdb_url database.
 
@@ -528,7 +629,7 @@ def connect_samdb(samdb_url, lp=None, session_info=None, credentials=None,
                  global_schema=global_schema)
 
 
-def connect_samdb_ex(samdb_url, lp=None, session_info=None, credentials=None,
+def connect_samdb_ex(samdb_url, *, lp=None, session_info=None, credentials=None,
                      flags=0, ldb_options=None, ldap_only=False):
     """Connects to samdb_url database
 
@@ -540,8 +641,9 @@ def connect_samdb_ex(samdb_url, lp=None, session_info=None, credentials=None,
     :param ldap_only: If set, only remote LDAP connection will be created.
     :return: (sam_db_connection, rootDse_record) tuple
     """
-    sam_db = connect_samdb(samdb_url, lp, session_info, credentials,
-                           flags, ldb_options, ldap_only)
+    sam_db = connect_samdb(samdb_url, lp=lp, session_info=session_info,
+                           credentials=credentials, flags=flags,
+                           ldb_options=ldb_options, ldap_only=ldap_only)
     # fetch RootDse
     res = sam_db.search(base="", expression="", scope=ldb.SCOPE_BASE,
                         attrs=["*"])
@@ -560,7 +662,7 @@ def connect_samdb_env(env_url, env_username, env_password, lp=None):
     creds = credentials.Credentials()
     if lp is None:
         # guess Credentials parameters here. Otherwise workstation
-        # and domain fields are NULL and gencache code segfalts
+        # and domain fields are NULL and gencache code segfaults
         lp = param.LoadParm()
         creds.guess(lp)
     creds.set_username(env_get_var_value(env_username))
@@ -640,7 +742,7 @@ def parse_help_consistency(out,
         prev = ' '
         for c in line:
             if state == OptState.NOOPT:
-                if c == '-' and  prev.isspace():
+                if c == '-' and prev.isspace():
                     state = OptState.HYPHEN1
                 prev = c
                 continue
@@ -655,7 +757,7 @@ def parse_help_consistency(out,
                 if c.isalnum():
                     name = '--' + c
                     state = OptState.NAME
-                else: # WTF, perhaps '--' ending option list.
+                else:  # WTF, perhaps '--' ending option list.
                     state = OptState.NOOPT
                     prev = c
                 continue
@@ -704,3 +806,19 @@ def check_help_consistency(out,
 
     if errors:
         return "\n".join(errors)
+
+
+def get_env_dir(key):
+    """A helper to pull a directory name from the environment, used in
+    some tests that optionally write e.g. fuzz seeds into a directory
+    named in an environment variable.
+    """
+    dir = os.environ.get(key)
+    if dir is None:
+        return None
+
+    if not os.path.isdir(dir):
+        raise ValueError(
+            f"{key} should name an existing directory (got '{dir}')")
+
+    return dir
