@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.11
 # Unix SMB/CIFS implementation.
 # Copyright (C) Stefan Metzmacher 2020
 # Copyright (C) 2020 Catalyst.Net Ltd
@@ -50,7 +50,7 @@ class KdcTests(RawKerberosTest):
     """
 
     def setUp(self):
-        super(KdcTests, self).setUp()
+        super().setUp()
         self.do_asn1_print = global_asn1_print
         self.do_hexdump = global_hexdump
 
@@ -164,7 +164,7 @@ class KdcTests(RawKerberosTest):
         self.check_as_reply(rep)
 
         etype = rep['enc-part']['etype']
-        self.assertEquals(AES256_CTS_HMAC_SHA1_96, etype)
+        self.assertEqual(AES256_CTS_HMAC_SHA1_96, etype)
 
     def test_arc4_hmac_md5(self):
         creds = self.get_user_creds()
@@ -178,7 +178,7 @@ class KdcTests(RawKerberosTest):
         self.check_as_reply(rep)
 
         etype = rep['enc-part']['etype']
-        self.assertEquals(ARCFOUR_HMAC_MD5, etype)
+        self.assertEqual(ARCFOUR_HMAC_MD5, etype)
 
     def test_aes_rc4(self):
         creds = self.get_user_creds()
@@ -192,7 +192,7 @@ class KdcTests(RawKerberosTest):
         self.check_as_reply(rep)
 
         etype = rep['enc-part']['etype']
-        self.assertEquals(AES256_CTS_HMAC_SHA1_96, etype)
+        self.assertEqual(AES256_CTS_HMAC_SHA1_96, etype)
 
     def test_clock_skew(self):
         creds = self.get_user_creds()
