@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.11
 # Unix SMB/CIFS implementation.
 # Copyright (C) Stefan Metzmacher 2020
 #
@@ -37,7 +37,7 @@ global_hexdump = False
 class SimpleKerberosTests(RawKerberosTest):
 
     def setUp(self):
-        super(SimpleKerberosTests, self).setUp()
+        super().setUp()
         self.do_asn1_print = global_asn1_print
         self.do_hexdump = global_hexdump
 
