@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.11
 # Unix SMB/CIFS implementation. Tests for smb notify
 # Copyright (C) Björn Baumbach <bb@samba.org> 2020
 #
@@ -24,7 +24,6 @@ os.environ["PYTHONUNBUFFERED"] = "1"
 import samba
 import random
 from samba.tests import TestCase
-from samba import NTSTATUSError
 from samba import credentials
 from samba.ntstatus import NT_STATUS_NOTIFY_CLEANUP
 from samba.samba3 import libsmb_samba_internal as libsmb
@@ -37,7 +36,7 @@ test_dir = os.path.join('notify_test_%d' % random.randint(0, 0xFFFF))
 
 class SMBNotifyTests(TestCase):
     def setUp(self):
-        super(SMBNotifyTests, self).setUp()
+        super().setUp()
         self.server = samba.tests.env_get_var_value("SERVER")
 
         # create an SMB connection to the server
@@ -74,7 +73,7 @@ class SMBNotifyTests(TestCase):
         self.smb_conn_unpriv = libsmb.Conn(self.server, self.share, self.lp, creds_unpriv)
 
     def tearDown(self):
-        super(SMBNotifyTests, self).tearDown()
+        super().tearDown()
         try:
             self.smb_conn.deltree(test_dir)
         except:
