@@ -25,7 +25,6 @@ import time
 import pwd
 
 from samba import Ldb, registry
-from samba.param import LoadParm
 from samba.provision import provision, ProvisioningError, setsysvolacl
 from samba.provision.common import FILL_FULL
 from samba.samba3 import passdb
@@ -95,13 +94,12 @@ def import_sam_policy(samdb, policy, logger):
         logger.warn("Could not set account policy, (%s)", str(e))
 
 
-def add_posix_attrs(logger, samdb, sid, name, nisdomain, xid_type, home=None,
+def add_posix_attrs(logger, samdb, sid, nisdomain, xid_type, home=None,
                     shell=None, pgid=None):
     """Add posix attributes for the user/group
 
     :param samdb: Samba4 sam.ldb database
     :param sid: user/group sid
-    :param sid: user/group name
     :param nisdomain: name of the (fake) NIS domain
     :param xid_type: type of id (ID_TYPE_UID/ID_TYPE_GID)
     :param home: user homedir (Unix homepath)
@@ -499,9 +497,8 @@ def upgrade_from_samba3(samba3, logger, targetdir, session_info=None,
         machineacct = s3db.getsampwnam('%s$' % netbiosname)
     except passdb.error:
         machinerid = None
-        machinesid = None
     else:
-        machinesid, machinerid = machineacct.user_sid.split()
+        _, machinerid = machineacct.user_sid.split()
 
     # Export account policy
     logger.info("Exporting account policy")
@@ -779,7 +776,7 @@ Please fix this account before attempting to upgrade again
             if g.gid != -1:
                 add_group_from_mapping_entry(result.samdb, g, logger)
                 add_ad_posix_idmap_entry(result.samdb, g.sid, g.gid, "ID_TYPE_GID", logger)
-                add_posix_attrs(samdb=result.samdb, sid=g.sid, name=g.nt_name, nisdomain=domainname.lower(), xid_type="ID_TYPE_GID", logger=logger)
+                add_posix_attrs(samdb=result.samdb, sid=g.sid, nisdomain=domainname.lower(), xid_type="ID_TYPE_GID", logger=logger)
 
     except:
         # We need this, so that we do not give even more errors due to not cancelling the transaction
@@ -810,7 +807,7 @@ Please fix this account before attempting to upgrade again
             if (username in homes) and (homes[username] is not None) and \
                (username in shells) and (shells[username] is not None) and \
                (username in pgids) and (pgids[username] is not None):
-                add_posix_attrs(samdb=result.samdb, sid=userdata[username].user_sid, name=username, nisdomain=domainname.lower(), xid_type="ID_TYPE_UID", home=homes[username], shell=shells[username], pgid=pgids[username], logger=logger)
+                add_posix_attrs(samdb=result.samdb, sid=userdata[username].user_sid, nisdomain=domainname.lower(), xid_type="ID_TYPE_UID", home=homes[username], shell=shells[username], pgid=pgids[username], logger=logger)
 
     logger.info("Adding users to groups")
     # Start a new transaction (should speed this up a little, due to index churn)
@@ -843,7 +840,7 @@ Please fix this account before attempting to upgrade again
         logger.info("Administrator password has been set to password of user '%s'", admin_user)
 
     if result.server_role == "active directory domain controller":
-        setsysvolacl(result.samdb, result.paths.netlogon, result.paths.sysvol,
+        setsysvolacl(result.samdb, result.paths.sysvol,
                      result.paths.root_uid, result.paths.root_gid,
                      security.dom_sid(result.domainsid), result.names.dnsdomain,
                      result.names.domaindn, result.lp, use_ntvfs)
